--------------------------------------------------------------------------------
--   _                 _                                     
--  | |_   _ _ __   __| |_   _       ___  ___ _ ____   _____ 
--  | | | | | '_ \ / _` | | | |     / __|/ _ \ '__\ \ / / _ \
--  | | |_| | | | | (_| | |_| |  _  \__ \  __/ |   \ V /  __/
--  |_|\__,_|_| |_|\__,_|\__,_| (_) |___/\___|_|    \_/ \___|
--
--  Serve: connecting 'lundu.routes' with 'xavante'
--
--  Copyright (C) 2010, Marc Simpson
--  Licensed under GPLv3 (see LICENSE).
--------------------------------------------------------------------------------

require "lundu.routes"
require "xavante"
require "xavante.httpd"

lundu         = lundu or {}
lundu.config  = lundu.config or {}
lundu.xavante = lundu.xavante or {}

--------------------------------------------------------------------------------

local query_params = function(q)
  local t = {}
  if ( not q ) then return t end
  for p in string.gmatch(q, "\??([^\&]+)") do
    for k,v in string.gmatch(p, "([^=]+)=(.*)") do
      t[k] = v
    end
  end
  return t
end

lundu.routehandler = function(conf)
  return function (req, res, ...)
    local path    = req.parsed_url.path
    local query   = req.parsed_url.query
    local meth    = string.lower(req.cmd_mth)
    local params  = query_params(query)
    --
    if ( meth == "post" ) then
      local l       = req.headers["content-length"]
      local message = req.socket:receive(l)
      params = query_params(message)
    end
    --
    res.headers["Content-Type"] = "text/html"
    res.content                 = lundu.perform(meth, path, params)
    --
    if ( type(res.content) == "table" ) then
      local error_code = res.content.error
      if ( error_code ) then
        local errstr    = tostring(error_code)
        res.statusline  = "HTTP/1.1 " .. errstr
        res.content 
        = "<html><head><title>Error "   .. errstr 
        .. "</title></head><h1>Error "  .. errstr
        .. "</h1>(Request URL: <tt>"    .. path   .. "</tt>)</body></html>"
      end
    end
    -- table.print(req)
    return res
  end
end

--------------------------------------------------------------------------------

lundu.config.port = 3030

lundu.xavante = function() 
  local rules = {
    {
      match   = "",
      with    = lundu.routehandler,
      params  = {}
    }
  }
  local port = lundu.config.port or "3030"
  xavante.HTTP {
    server      = {  host = "*", port = port },
    defaultHost = { rules = rules }
  }
  xavante.start_message(function() print("~ lundu started on port " .. port .. " ~") end)
  xavante.start()
end

--------------------------------------------------------------------------------
